# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:17+00:00

from __future__ import annotations

from typing import Any

from pydantic import RootModel


class Field0(RootModel[Any]):
    root: Any
