# generated by datamodel-codegen:
#   filename:  swagger.yaml
#   timestamp: 2024-07-12T07:32:22+00:00

from __future__ import annotations

from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel


class Secret(BaseModel):
    id: Optional[str] = None
    key: Optional[str] = None
    value: Optional[str] = None
    createdAt: Optional[datetime] = None
    enabled: Optional[bool] = None


class PairStringString(BaseModel):
    value: Optional[str] = None
    key: Optional[str] = None
    left: Optional[str] = None
    right: Optional[str] = None


class ConnectedOuterService(BaseModel):
    name: Optional[str] = None
    version: Optional[str] = None


class VersionInfo(BaseModel):
    groupId: Optional[str] = None
    artifactId: Optional[str] = None
    version: Optional[str] = None
    buildTime: Optional[str] = None
    branch: Optional[str] = None
    commitId: Optional[str] = None
    commitMessage: Optional[str] = None
    connectedOuterServices: Optional[List[ConnectedOuterService]] = None


class ListSecretsResponse(BaseModel):
    secrets: Optional[List[Secret]] = None
    notFoundSecrets: Optional[List[PairStringString]] = None
